// File:        reporter.H

#ifndef REPORTER_H
#define REPORTER_H

//============================================================================
// Content:         class HReporter
// Compilers:       Borland C++ 4.0 and Microsoft Visual C++ 1.5
// Memory Mode:     Large
// Author:          James Huang  (74577,2136@compuserve.com)
// Date:            Nov. 1994
// Distribution:    Freeware
// Warranty:        As-is
//============================================================================
//
// Description:
//
//  This class is a C++ wrapper for all Crystal Reports Printing Engine(tm)
//  (CRPE) APIs. In particular, it simplifies report printing and previewing.
//  To do more sophisticated operations upon an open report, such as print-
//  time customization, export to files/e-mail, etc., use the CRPE-API-
//  counterpart member functions. (Refer to the CRPE documents for details or
//  on-line help; you have to understand them, anyway).
//
//  An object of HReporter is able to handle one report at a time.
//  To use a report, open it first by calling OpenReport(). The last open
//  report is automatically closed, so is its preview window, if it was on.
//  At destruction time, the open report and its preview window, if any, are
//  closed. (Therefore, be careful about the object's scope to ensure correct
//  preview window displaying!) Normally an instance of this class is created
//  in your App class or main window class. If you need to handle more than
//  one report at a time, create multiple instances for each report.
//
//  To print the report, simply call DoPrint().
//  To preview the report, call DoMDIPreview() to do previewing in an MDI
//  application, or call OutputToWindow() to customize the preview window.
//
//  Most CRPE APIs requires a print job handle parameter; all these functions
//  are enclosed in HReporter without it. Also, CRPE APIs returning variable
//  strings now return a LPSTR pointing to a buffer. Use member functions as
//  much as possible. The only CRPE APIs not covered are
//          PEGetVersion (short versionRequested)
//          PELogOnServer (LPSTR dllName, struct PELogOnInfo*)
//          PELogOffServer (LPSTR dllName, struct PELogOnInfo*)
//
//============================================================================
//
// Tips for using CRPE: (if you are new to CRPE)
//
//  At build time, have CRPE.H available, and link in CRPE.LIB.
//  At run time, always make sure that
//  1) The correct version of CRPE.DLL and its related DLLs are hit;
//  2) The correct report file is loaded; and
//  3) The data sources for the report are accessible from your environment
//     (such as keep your local database tables in the same directory as the
//      report, then specify data source location "same as report".)
//
//============================================================================

#define  STRICT
#include <windows.h>
#include "crpe.h"

#define  MAXTITLELEN    60


class HReporter {   // All functions with BOOL return type of no Boolean
					// meaning return TRUE for OK and FALSE for error.
public:
	HReporter (LPSTR reportPath=NULL); // the report is opened if specified.
	~HReporter ();

	// Printing Engine and Report Handling ...
	void    EnableErrorHandling (BOOL doerror=TRUE) {doError=doerror;}
		// when error handling is enabled, a message box is popped up for
		// errors in functions that open, close, print and preview reports;
		// errors for other CRPE-counterpart functions are not handled.
	BOOL    IsEngineOpened ()   {return CRPE_Count>0;}
	BOOL    OpenReport (LPSTR reportPath);  // closes last open job.
	BOOL    CanCloseReport ();
	BOOL    CloseReport ();

	// Printing the report ...
	BOOL    DoPrint (int nCopies=1);

	// Previewing the report ...
	BOOL    OutputToWindow (LPSTR title=0,HWND parent=0,LONG style=0,
							short l=CW_USEDEFAULT,short t=CW_USEDEFAULT,
							short w=CW_USEDEFAULT,short h=CW_USEDEFAULT);
			// With all default values, a top-level preview window is created
			// with frame, caption (of "Report Preview"), etc.
	BOOL    DoMDIPreview (HWND hClient,LPSTR title=0);
			// An MDI child is created, and the preview window is a child
			// occupying its whole client area.
	BOOL    CloseWindow ();
	BOOL    IsWindowOn () {return HWindow()!=0;}
	HWND    HWindow () {return wndAttr.HWindow();}

	///////////////////////////////////////////////////////////////////////////
	//
	//  The following member funcitons are CRPE API counterparts. (All inline)
	//  What is done (to the original APIs) here are:
	//
	//  1) Get rid of the PE prefix and the first 'short printJob' parameter;
	//  2) For functions obtaining a string, returns a LPSTR; or NULL if error.
	//     (the returned LPSTR should be handled like a LPCSTR.)
	//
	//  For more details about each function, refer to CR documents.
	//
	///////////////////////////////////////////////////////////////////////////

	LPSTR GetErrorText ();
	BOOL  IsPrintJobFinished ();
	BOOL  GetJobStatus (PEJobInfo* jobInfo);
	BOOL  SetPrintDate (short year, short month, short day);
	LPSTR GetFormula (LPSTR formulaName);
	BOOL  SetFormula (LPSTR formulaName, LPSTR formulaString);
	LPSTR GetSelectionFormula ();
	BOOL  SetSelectionFormula (LPSTR formulaString);
	LPSTR GetGroupSelectionFormula ();
	BOOL  SetGroupSelectionFormula (LPSTR formulaString);
	BOOL  SetGroupCondition (short sectionCode, LPSTR conditionField,
								short condition, short sortDirection);
	short GetNSortFields ();
	LPSTR GetNthSortField (short sortFieldN, short* direction);
	BOOL  SetNthSortField (short sortFieldN, LPSTR name, short direction);
	BOOL  DeleteNthSortField (short sortFieldN);
	short GetNGroupSortFields ();
	LPSTR GetNthGroupSortField (short sortFieldN, short* direction);
	BOOL  SetNthGroupSortField (short sortFieldN, LPSTR name, short direction);
	BOOL  DeleteNthGroupSortField (short sortFieldN);
	short GetNTables ();
	BOOL  GetNthTableType (short tableN, PETableType* tableType);
	BOOL  GetNthTableSessionInfo (short tableN, PESessionInfo* sessionInfo);
	BOOL  SetNthTableSessionInfo (short tableN,PESessionInfo* sessionInfo,
								BOOL propagateAcrossTables);
	BOOL  GetNthTableLogOnInfo (short tableN, PELogOnInfo* logOnInfo);
	BOOL  SetNthTableLogOnInfo (short tableN, PELogOnInfo* logOnInfo,
								BOOL propagateAcrossTables);
	BOOL  GetNthTableLocation (short tableN, PETableLocation* location);
	BOOL  SetNthTableLocation (short tableN, PETableLocation* location);
	BOOL  TestNthTableConnectivity (short tableN);
	short GetNFiles ();
	LPSTR GetNthFileName (short fileN);
	BOOL  SetNthFileName (short fileN, LPSTR name);
	LPSTR GetSQLQuery ();
	BOOL  SetSQLQuery (LPSTR queryString);
	BOOL  SetRefreshData (BOOL refreshData);
	LPSTR GetReportTitle ();
	BOOL  SetReportTitle (LPSTR title);
	BOOL  ShowNextPage ();
	BOOL  ShowFirstPage ();
	BOOL  ShowPreviousPage ();
	BOOL  ShowLastPage ();
	BOOL  ShowPrintControls (BOOL showPrintControls);
	BOOL  SelectPrinter (LPSTR driverName, LPSTR printerName, LPSTR portName,
								DEVMODE* mode);
	BOOL  OutputToDefaultPrinter (short nCopies);
	BOOL  SetNDetailCopies (short nDetailCopies);
	BOOL  SetPrintOptions (PEPrintOptions FAR *options);
	BOOL  OutputToFile (LPSTR outputFilePath, short type, LPVOID options);
	BOOL  GetExportOptions (PEExportOptions* options);
	BOOL  ExportTo (PEExportOptions* options);
	BOOL  SetMargins (short left, short right, short top, short bottom);
	BOOL  SetMinimumSectionHeight (short sectionCode, short minimumHeight);
	BOOL  SetSectionFormat (short sectionCode, PESectionOptions* options);
	BOOL  SetLineHeight (short sctnCode, short lineN, short height, short ascent);
	BOOL  SetFont (short sectionCode, short scopeCode, LPSTR faceName,
			short fontFamily, short fontPitch, short charSet, short pointSize,
			short isItalic, short isUnderlined, short isStruckOut, short weight);

// Implementation

protected:  // data
	// operational status --
	short   printJob;
	BOOL    doError;

	// Preview creation parameters --
	struct WindowAttr {
		HWND    hWindow;    // set by OutputToWindow() if successful.
		HWND    hParent;    // Set only by MDIPreview(), not OutputToWindow().
		// functions --
		WindowAttr () {Clear();}
		void    Clear () {hWindow=hParent=0;}
		HWND    CreateMDIChild (HWND hClient,LPSTR title=0);
		HWND    HWindow () {return hParent?hParent:hWindow;}
	} wndAttr;

protected:  // functions
	void  RegisterToEngine ();
	BOOL  GetHandleString (); // return string into 'text'; memory allocated.
	void  Error (LPCSTR msg);

private: // data
	// engine status --
	static short CRPE_Count;
	// variable-length string handling --
	HANDLE  handle;
	short   len;
	LPSTR   text;

};  // end of class HReporter.


//===========================================================================
// Inline function implementation --

inline LPSTR HReporter::GetErrorText ()
{return PEGetErrorText(printJob,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::IsPrintJobFinished ()
{return PEIsPrintJobFinished(printJob);}

inline BOOL  HReporter::GetJobStatus (PEJobInfo* jobInfo)
{return PEGetJobStatus(printJob,jobInfo);}

inline BOOL  HReporter::SetPrintDate (short year, short month, short day)
{return PESetPrintDate(printJob,year,month,day);}

inline LPSTR HReporter::GetFormula (LPSTR fml)
{return PEGetFormula(printJob,fml,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetFormula (LPSTR formulaName, LPSTR formulaString)
{return PESetFormula(printJob,formulaName,formulaString);}

inline LPSTR HReporter::GetSelectionFormula ()
{return PEGetSelectionFormula(printJob,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetSelectionFormula (LPSTR formulaString)
{return PESetSelectionFormula(printJob,formulaString);}

inline LPSTR HReporter::GetGroupSelectionFormula ()
{return PEGetGroupSelectionFormula(printJob,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetGroupSelectionFormula (LPSTR formulaString)
{return PESetGroupSelectionFormula(printJob,formulaString);}

inline BOOL  HReporter::SetGroupCondition (short a, LPSTR b, short c, short d)
{return PESetGroupCondition(printJob,a,b,c,d);}

inline short HReporter::GetNSortFields ()
{return PEGetNSortFields(printJob):0;}

inline LPSTR HReporter::GetNthSortField (short sortFieldN, short* direction)
{return PEGetNthSortField(printJob,sortFieldN,&handle,&len,direction)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetNthSortField (short a, LPSTR b, short c)
{return PESetNthSortField(printJob,a,b,c);}

inline BOOL  HReporter::DeleteNthSortField (short sortFieldN)
{return PEDeleteNthSortField(printJob,sortFieldN);}

inline short HReporter::GetNGroupSortFields ()
{return PEGetNGroupSortFields(printJob);}

inline LPSTR HReporter::GetNthGroupSortField (short sortFieldN, short* direction)
{return PEGetNthGroupSortField(printJob,sortFieldN,&handle,&len,direction)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetNthGroupSortField (short a, LPSTR b, short c)
{return PESetNthGroupSortField(printJob,a,b,c);}

inline BOOL  HReporter::DeleteNthGroupSortField (short sortFieldN)
{return PEDeleteNthGroupSortField(printJob,sortFieldN);}

inline short HReporter::GetNTables ()
{return PEGetNTables(printJob);}

inline LPSTR HReporter::GetNthFileName (short fileN)
{return PEGetNthFileName(printJob,fileN,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::GetNthTableType (short a,PETableType* b)
{return PEGetNthTableType(printJob,a,b);}

inline BOOL  HReporter::GetNthTableSessionInfo (short a,PESessionInfo* b)
{return PEGetNthTableSessionInfo(printJob,a,b);}

inline BOOL  HReporter::SetNthTableSessionInfo (short a,PESessionInfo* b,BOOL c)
{return PESetNthTableSessionInfo(printJob,a,b,c);}

inline BOOL  HReporter::GetNthTableLogOnInfo (short a,PELogOnInfo* b)
{return PEGetNthTableLogOnInfo(printJob,a,b);}

inline BOOL  HReporter::SetNthTableLogOnInfo (short a,PELogOnInfo* b,BOOL c)
{return PESetNthTableLogOnInfo(printJob,a,b,c);}

inline BOOL  HReporter::GetNthTableLocation (short a,PETableLocation* b)
{return PEGetNthTableLocation(printJob,a,b);}

inline BOOL  HReporter::SetNthTableLocation (short a,PETableLocation* b)
{return PESetNthTableLocation(printJob,a,b);}

inline BOOL  HReporter::TestNthTableConnectivity (short tableN)
{return PETestNthTableConnectivity(printJob,tableN);}

inline short HReporter::GetNFiles ()
{return PEGetNFiles(printJob):0;}

inline BOOL  HReporter::SetNthFileName (short fileN, LPSTR name)
{return PESetNthFileName(printJob,fileN,name);}

inline LPSTR HReporter::GetSQLQuery ()
{return PEGetSQLQuery(printJob,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetSQLQuery (LPSTR queryString)
{return PESetSQLQuery(printJob,queryString);}

inline BOOL  HReporter::SetRefreshData (BOOL refreshData)
{return PESetRefreshData(printJob,refreshData);}

inline LPSTR HReporter::GetReportTitle ()
{return PEGetReportTitle(printJob,&handle,&len)&&GetHandleString()?text:NULL;}

inline BOOL  HReporter::SetReportTitle (LPSTR title)
{return PESetReportTitle(printJob,title);}

inline BOOL  HReporter::ShowNextPage ()
{return PEShowNextPage(printJob);}

inline BOOL  HReporter::ShowFirstPage ()
{return PEShowFirstPage(printJob);}

inline BOOL  HReporter::ShowPreviousPage ()
{return PEShowPreviousPage(printJob);}

inline BOOL  HReporter::ShowLastPage ()
{return PEShowLastPage(printJob);}

inline BOOL  HReporter::ShowPrintControls (BOOL showPrintControls)
{return PEShowPrintControls(printJob,showPrintControls);}

inline BOOL  HReporter::SelectPrinter (LPSTR a, LPSTR b, LPSTR c, DEVMODE* d)
{return PESelectPrinter(printJob,a,b,c,d);}

inline BOOL  HReporter::OutputToDefaultPrinter (short nCopies)
{return PEOutputToDefaultPrinter(printJob,nCopies);}

inline BOOL  HReporter::SetNDetailCopies (short nDetailCopies)
{return PESetNDetailCopies(printJob,nDetailCopies);}

inline BOOL  HReporter::SetPrintOptions (PEPrintOptions* options)
{return PESetPrintOptions(printJob,options);}

inline BOOL  HReporter::OutputToFile (LPSTR a, short b, LPVOID c)
{return PEOutputToFile(printJob,a,b,c);}

inline BOOL  HReporter::GetExportOptions (PEExportOptions* options)
{return PEGetExportOptions(printJob,options);}

inline BOOL  HReporter::ExportTo (PEExportOptions* options)
{return PEExportTo(printJob,options);}

inline BOOL  HReporter::SetMargins (short a, short b, short c, short d)
{return PESetMargins(printJob,a,b,c,d);}

inline BOOL  HReporter::SetMinimumSectionHeight (short a, short b)
{return PESetMinimumSectionHeight(printJob,a,b);}

inline BOOL  HReporter::SetSectionFormat (short a, PESectionOptions* b)
{return PESetSectionFormat(printJob,a,b);}

inline BOOL  HReporter::SetLineHeight (short a, short b, short c, short d)
{return PESetLineHeight(printJob,a,b,c,d);}

inline BOOL  HReporter::SetFont (short a,short b,LPSTR c,short d,short e,short f,short g,short h,short i,short j,short k)
{return PESetFont(printJob,a,b,c,d,e,f,g,h,i,j,k);}

#endif
